/*
 * share_context.c:
 * Simple ShareContext with GdkGLPixmap example.
 *
 * written by Roberto Huelga  <rhuelga@gmail.com>
 *	based on sample by Naofumi Yasufuku  <naofumi@users.sourceforge.net>
 */

#include <stdlib.h>
#include <stdio.h>

#include <gtk/gtk.h>

#include <gtk/gtkgl.h>

#include <GL/gl.h>
#include <GL/glu.h>


GdkPixmap *	pixmap;
GdkGLContext	*global_context;

static void
realize (GtkWidget *widget,
         gpointer   data)
{
	GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);


	static GLfloat light_diffuse[] = {1.0, 0.0, 0.0, 1.0};
	static GLfloat light_position[] = {1.0, 1.0, 1.0, 0.0};

	/*** OpenGL BEGIN ***/
	if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
		return;

	glLightfv (GL_LIGHT0, GL_DIFFUSE, light_diffuse);
	glLightfv (GL_LIGHT0, GL_POSITION, light_position);
	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);
	glEnable (GL_DEPTH_TEST);

	glClearColor (1.0, 1.0, 1.0, 1.0);
	glClearDepth (1.0);

	glViewport (0, 0,
		    widget->allocation.width, widget->allocation.height);

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective (40.0, 1.0, 1.0, 10.0);

	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	gluLookAt (0.0, 0.0, 3.0,
		   0.0, 0.0, 0.0,
		   0.0, 1.0, 0.0);
	glTranslatef (0.0, 0.0, -3.0);

	gdk_gl_drawable_gl_end (gldrawable);
	/*** OpenGL END ***/
}

static gboolean
configure_event (GtkWidget         *widget,
                 GdkEventConfigure *event,
                 gpointer           data)
{
	GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

	/*** OpenGL BEGIN ***/
	if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
		return FALSE;

	glViewport (0, 0,
		    widget->allocation.width, widget->allocation.height);

	gdk_gl_drawable_gl_end (gldrawable);
	/*** OpenGL END ***/

	return TRUE;
}

static gboolean
expose_event (GtkWidget      *widget,
              GdkEventExpose *event,
              gpointer        data)
{
	GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

	/*** OpenGL BEGIN ***/
	if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
		return FALSE;

	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if ( glIsList( 1 ) ) {
		glCallList (1);
	} else {
		g_printerr( "List not shared\n");
	}
	

	if (gdk_gl_drawable_is_double_buffered (gldrawable))
		gdk_gl_drawable_swap_buffers (gldrawable);
	else
		glFlush ();

	gdk_gl_drawable_gl_end (gldrawable);
	/*** OpenGL END ***/

	return TRUE;
}

int
main (int   argc,
      char *argv[])
{
	GdkGLConfig *glconfig;
	gint major, minor;

	GtkWidget *window;
	GtkWidget *vbox;
	GtkWidget *drawing_area;
	GtkWidget *button;

	/*
	 * Init GTK.
	 */

	gtk_init (&argc, &argv);

	/*
	 * Init GtkGLExt.
	 */

	gtk_gl_init (&argc, &argv);

	/*
	 * Query OpenGL extension version.
	 */

	gdk_gl_query_version (&major, &minor);
	g_print ("\nOpenGL extension version - %d.%d\n\n",
		 major, minor);

	/*
	 * Configure OpenGL-capable visual.
	 */

	/* Try double-buffered visual */
	glconfig = gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB |
					      GDK_GL_MODE_DEPTH |
					      GDK_GL_MODE_DOUBLE);
	if (glconfig == NULL)
		{
			g_print ("*** Cannot find the double-buffered visual.\n");
			g_print ("*** Trying single-buffered visual.\n");

			/* Try single-buffered visual */
			glconfig = gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB |
							      GDK_GL_MODE_DEPTH);
			if (glconfig == NULL)
				{
					g_print ("*** No appropriate OpenGL-capable visual found.\n");
					exit (1);
				}
		}


	/*
	 * GL Pixmap
	 */
	pixmap = gdk_pixmap_new( NULL, 256, 256, 24 );
	GdkGLPixmap *gl_pixmap = gdk_pixmap_set_gl_capability( pixmap,
							       glconfig,
							       NULL );
	GdkGLDrawable * gl_drawable = gdk_pixmap_get_gl_drawable( GDK_PIXMAP( pixmap ));
	global_context = gdk_gl_context_new( gl_drawable,
					     NULL,
					     FALSE,
					     GDK_GL_RGBA_TYPE );

	if ( !gdk_gl_drawable_gl_begin( gl_drawable, global_context ) ) {
		g_printerr( "the offscreen opengl context don't work\n");
		exit( -1 );
	}
	
	GLUquadricObj *qobj;
	qobj = gluNewQuadric ();
	gluQuadricDrawStyle (qobj, GLU_FILL);
	glNewList (1, GL_COMPILE);
	gluSphere (qobj, 1.0, 20, 20);
	glEndList ();

	
	gdk_gl_drawable_gl_end( gl_drawable );
	
	

	/*
	 * Top-level window.
	 */

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window), "simple");

	/* Get automatically redrawn if any of their children changed allocation. */
	gtk_container_set_reallocate_redraws (GTK_CONTAINER (window), TRUE);

	g_signal_connect (G_OBJECT (window), "delete_event",
			  G_CALLBACK (gtk_main_quit), NULL);

	/*
	 * VBox.
	 */

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (window), vbox);
	gtk_widget_show (vbox);

	/*
	 * Drawing area for drawing OpenGL scene.
	 */

	drawing_area = gtk_drawing_area_new ();
	gtk_widget_set_size_request (drawing_area, 200, 200);

	/* Set OpenGL-capability to the widget. */
	gtk_widget_set_gl_capability (drawing_area,
				      glconfig,
				      global_context,
				      TRUE,
				      GDK_GL_RGBA_TYPE);

	gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);

	gtk_widget_set_events (drawing_area,
			       GDK_EXPOSURE_MASK |
			       GDK_BUTTON_PRESS_MASK);

	g_signal_connect_after (G_OBJECT (drawing_area), "realize",
				G_CALLBACK (realize), NULL);
	g_signal_connect (G_OBJECT (drawing_area), "configure_event",
			  G_CALLBACK (configure_event), NULL);
	g_signal_connect (G_OBJECT (drawing_area), "expose_event",
			  G_CALLBACK (expose_event), NULL);

	gtk_widget_show (drawing_area);


	/*
	 * Simple quit button.
	 */

	button = gtk_button_new_with_label ("Quit");
	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);

	g_signal_connect (G_OBJECT (button), "clicked",
			  G_CALLBACK (gtk_main_quit), NULL);

	gtk_widget_show (button);

	/*
	 * Show window.
	 */

	gtk_widget_show (window);

	/*
	 * Main loop.
	 */

	gtk_main ();

	return 0;
}
